pub struct StorageDoubleMap<Prefix, Hasher1, Key1, Hasher2, Key2, Value, QueryKind = OptionQuery, OnEmpty = GetDefault, MaxValues = GetDefault>(/* private fields */);Expand description
A type representing a double map in storage. This structure associates a pair of keys with a value of a specified type stored on-chain.
A double map with keys k1 and k2 can be likened to a
StorageMap with a key of type (k1, k2).
However, a double map offers functions specific to each key, enabling partial iteration and
deletion based on one key alone.
Also, conceptually, a double map is a special case of a
StorageNMap using two keys.
For general information regarding the #[pallet::storage] attribute, refer to
crate::pallet_macros::storage.
Examples
Kitchen-sink
#[frame_support::pallet]
mod pallet {
/// A kitchen-sink StorageDoubleMap, with all possible additional attributes.
#[pallet::storage]
#[pallet::getter(fn foo)]
#[pallet::storage_prefix = "OtherFoo"]
#[pallet::unbounded]
pub type Foo<T> = StorageDoubleMap<
_,
Blake2_128Concat,
u8,
Twox64Concat,
u16,
u32,
ValueQuery
>;
/// Alternative named syntax.
#[pallet::storage]
pub type Bar<T> = StorageDoubleMap<
Hasher1 = Blake2_128Concat,
Key1 = u8,
Hasher2 = Twox64Concat,
Key2 = u16,
Value = u32,
QueryKind = ValueQuery
>;
}Partial Iteration & Removal
When Hasher1 and Hasher2 implement the
ReversibleStorageHasher trait, the first key k1
can be used to partially iterate over keys and values of the double map, and to delete items.
#[test]
fn example_double_map_partial_operations() {
type FooDoubleMap =
StorageDoubleMap<Prefix, Blake2_128Concat, u32, Blake2_128Concat, u32, u32, ValueQuery>;
TestExternalities::default().execute_with(|| {
FooDoubleMap::insert(0, 0, 42);
FooDoubleMap::insert(0, 1, 43);
FooDoubleMap::insert(1, 0, 314);
// should be equal to {0,1} (ordering is random)
let collected_k2_keys: BTreeSet<_> = FooDoubleMap::iter_key_prefix(0).collect();
assert_eq!(collected_k2_keys, [0, 1].iter().copied().collect::<BTreeSet<_>>());
// should be equal to {42,43} (ordering is random)
let collected_k2_values: BTreeSet<_> = FooDoubleMap::iter_prefix_values(0).collect();
assert_eq!(collected_k2_values, [42, 43].iter().copied().collect::<BTreeSet<_>>());
// Remove items from the map using k1 = 0
let _ = FooDoubleMap::clear_prefix(0, u32::max_value(), None);
// Values associated with (0, _) should have been removed
assert_eq!(FooDoubleMap::iter_prefix(0).collect::<Vec<_>>(), vec![]);
});
}Implementations
Sourceimpl<Prefix, Hasher1, Key1, Hasher2, Key2, Value, QueryKind, OnEmpty, MaxValues> StorageDoubleMap<Prefix, Hasher1, Key1, Hasher2, Key2, Value, QueryKind, OnEmpty, MaxValues>where
Prefix: StorageInstance,
Hasher1: StorageHasher,
Hasher2: StorageHasher,
Key1: FullCodec,
Key2: FullCodec,
Value: FullCodec,
QueryKind: QueryKindTrait<Value, OnEmpty>,
OnEmpty: Get<QueryKind::Query> + 'static,
MaxValues: Get<Option<u32>>,
impl<Prefix, Hasher1, Key1, Hasher2, Key2, Value, QueryKind, OnEmpty, MaxValues> StorageDoubleMap<Prefix, Hasher1, Key1, Hasher2, Key2, Value, QueryKind, OnEmpty, MaxValues>where
Prefix: StorageInstance,
Hasher1: StorageHasher,
Hasher2: StorageHasher,
Key1: FullCodec,
Key2: FullCodec,
Value: FullCodec,
QueryKind: QueryKindTrait<Value, OnEmpty>,
OnEmpty: Get<QueryKind::Query> + 'static,
MaxValues: Get<Option<u32>>,
Sourcepub fn hashed_key_for<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> Vec<u8> ⓘwhere
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
pub fn hashed_key_for<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> Vec<u8> ⓘwhere
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Get the storage key used to fetch a value corresponding to a specific key.
Sourcepub fn contains_key<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> boolwhere
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
pub fn contains_key<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> boolwhere
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Does the value (explicitly) exist in storage?
Sourcepub fn get<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> QueryKind::Querywhere
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
pub fn get<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> QueryKind::Querywhere
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Load the value associated with the given key from the double map.
Sourcepub fn try_get<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> Result<Value, ()>where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
pub fn try_get<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> Result<Value, ()>where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Try to get the value for the given key from the double map.
Returns Ok if it exists, Err if not.
Sourcepub fn set<KArg1: EncodeLike<Key1>, KArg2: EncodeLike<Key2>>(
k1: KArg1,
k2: KArg2,
q: QueryKind::Query,
)
pub fn set<KArg1: EncodeLike<Key1>, KArg2: EncodeLike<Key2>>( k1: KArg1, k2: KArg2, q: QueryKind::Query, )
Store or remove the value to be associated with key so that get returns the query.
Sourcepub fn take<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> QueryKind::Querywhere
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
pub fn take<KArg1, KArg2>(k1: KArg1, k2: KArg2) -> QueryKind::Querywhere
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Take a value from storage, removing it afterwards.
Sourcepub fn swap<XKArg1, XKArg2, YKArg1, YKArg2>(
x_k1: XKArg1,
x_k2: XKArg2,
y_k1: YKArg1,
y_k2: YKArg2,
)where
XKArg1: EncodeLike<Key1>,
XKArg2: EncodeLike<Key2>,
YKArg1: EncodeLike<Key1>,
YKArg2: EncodeLike<Key2>,
pub fn swap<XKArg1, XKArg2, YKArg1, YKArg2>(
x_k1: XKArg1,
x_k2: XKArg2,
y_k1: YKArg1,
y_k2: YKArg2,
)where
XKArg1: EncodeLike<Key1>,
XKArg2: EncodeLike<Key2>,
YKArg1: EncodeLike<Key1>,
YKArg2: EncodeLike<Key2>,
Swap the values of two key-pairs.
Sourcepub fn insert<KArg1, KArg2, VArg>(k1: KArg1, k2: KArg2, val: VArg)where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
VArg: EncodeLike<Value>,
pub fn insert<KArg1, KArg2, VArg>(k1: KArg1, k2: KArg2, val: VArg)where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
VArg: EncodeLike<Value>,
Store a value to be associated with the given keys from the double map.
Sourcepub fn remove<KArg1, KArg2>(k1: KArg1, k2: KArg2)where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
pub fn remove<KArg1, KArg2>(k1: KArg1, k2: KArg2)where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Remove the value under the given keys.
Sourcepub fn remove_prefix<KArg1>(k1: KArg1, limit: Option<u32>) -> KillStorageResultwhere
KArg1: ?Sized + EncodeLike<Key1>,
👎Deprecated: Use clear_prefix instead
pub fn remove_prefix<KArg1>(k1: KArg1, limit: Option<u32>) -> KillStorageResultwhere
KArg1: ?Sized + EncodeLike<Key1>,
clear_prefix insteadRemove all values under k1 in the overlay and up to limit in the
backend.
All values in the client overlay will be deleted, if there is some limit then up to
limit values are deleted from the client backend, if limit is none then all values in
the client backend are deleted.
Note
Calling this multiple times per block with a limit set leads always to the same keys being
removed and the same result being returned. This happens because the keys to delete in the
overlay are not taken into account when deleting keys in the backend.
Sourcepub fn clear_prefix<KArg1>(
first_key: KArg1,
limit: u32,
maybe_cursor: Option<&[u8]>,
) -> MultiRemovalResultswhere
KArg1: ?Sized + EncodeLike<Key1>,
pub fn clear_prefix<KArg1>(
first_key: KArg1,
limit: u32,
maybe_cursor: Option<&[u8]>,
) -> MultiRemovalResultswhere
KArg1: ?Sized + EncodeLike<Key1>,
Attempt to remove items from the map matching a first_key prefix.
Returns MultiRemovalResults to inform about the result. Once
the resultant maybe_cursor field is None, then no further items remain to be deleted.
NOTE: After the initial call for any given map, it is important that no further items
are inserted into the map which match the first_key. If so, then the map may not be
empty when the resultant maybe_cursor is None.
Limit
A limit must always be provided through in order to cap the maximum
amount of deletions done in a single call. This is one fewer than the
maximum number of backend iterations which may be done by this operation and as such
represents the maximum number of backend deletions which may happen. A limit of zero
implies that no keys will be deleted, though there may be a single iteration done.
Cursor
A cursor may be passed in to this operation with maybe_cursor. None should only be
passed once (in the initial call) for any given storage map and first_key. Subsequent
calls operating on the same map/first_key should always pass Some, and this should be
equal to the previous call result’s maybe_cursor field.
Sourcepub fn iter_prefix_values<KArg1>(k1: KArg1) -> PrefixIterator<Value> ⓘwhere
KArg1: ?Sized + EncodeLike<Key1>,
pub fn iter_prefix_values<KArg1>(k1: KArg1) -> PrefixIterator<Value> ⓘwhere
KArg1: ?Sized + EncodeLike<Key1>,
Iterate over values that share the first key.
Sourcepub fn mutate<KArg1, KArg2, R, F>(k1: KArg1, k2: KArg2, f: F) -> R
pub fn mutate<KArg1, KArg2, R, F>(k1: KArg1, k2: KArg2, f: F) -> R
Mutate the value under the given keys.
Sourcepub fn try_mutate<KArg1, KArg2, R, E, F>(
k1: KArg1,
k2: KArg2,
f: F,
) -> Result<R, E>
pub fn try_mutate<KArg1, KArg2, R, E, F>( k1: KArg1, k2: KArg2, f: F, ) -> Result<R, E>
Mutate the value under the given keys when the closure returns Ok.
Sourcepub fn mutate_exists<KArg1, KArg2, R, F>(k1: KArg1, k2: KArg2, f: F) -> R
pub fn mutate_exists<KArg1, KArg2, R, F>(k1: KArg1, k2: KArg2, f: F) -> R
Mutate the value under the given keys. Deletes the item if mutated to a None.
Sourcepub fn try_mutate_exists<KArg1, KArg2, R, E, F>(
k1: KArg1,
k2: KArg2,
f: F,
) -> Result<R, E>
pub fn try_mutate_exists<KArg1, KArg2, R, E, F>( k1: KArg1, k2: KArg2, f: F, ) -> Result<R, E>
Mutate the item, only if an Ok value is returned. Deletes the item if mutated to a None.
f will always be called with an option representing if the storage item exists (Some<V>)
or if the storage item does not exist (None), independent of the QueryType.
Sourcepub fn append<Item, EncodeLikeItem, KArg1, KArg2>(
k1: KArg1,
k2: KArg2,
item: EncodeLikeItem,
)where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Item: Encode,
EncodeLikeItem: EncodeLike<Item>,
Value: StorageAppend<Item>,
pub fn append<Item, EncodeLikeItem, KArg1, KArg2>(
k1: KArg1,
k2: KArg2,
item: EncodeLikeItem,
)where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Item: Encode,
EncodeLikeItem: EncodeLike<Item>,
Value: StorageAppend<Item>,
Append the given item to the value in the storage.
Value is required to implement StorageAppend.
Warning
If the storage item is not encoded properly, the storage will be overwritten
and set to [item]. Any default value set for the storage item will be ignored
on overwrite.
Sourcepub fn decode_len<KArg1, KArg2>(key1: KArg1, key2: KArg2) -> Option<usize>where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Value: StorageDecodeLength,
pub fn decode_len<KArg1, KArg2>(key1: KArg1, key2: KArg2) -> Option<usize>where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Value: StorageDecodeLength,
Read the length of the storage value without decoding the entire value under the
given key1 and key2.
Value is required to implement StorageDecodeLength.
If the value does not exists or it fails to decode the length, None is returned.
Otherwise Some(len) is returned.
Warning
None does not mean that get() does not return a value. The default value is completely
ignored by this function.
Sourcepub fn decode_non_dedup_len<KArg1, KArg2>(
key1: KArg1,
key2: KArg2,
) -> Option<usize>where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Value: StorageDecodeNonDedupLength,
pub fn decode_non_dedup_len<KArg1, KArg2>(
key1: KArg1,
key2: KArg2,
) -> Option<usize>where
KArg1: EncodeLike<Key1>,
KArg2: EncodeLike<Key2>,
Value: StorageDecodeNonDedupLength,
Read the length of the storage value without decoding the entire value.
Value is required to implement StorageDecodeNonDedupLength.
If the value does not exists or it fails to decode the length, None is returned.
Otherwise Some(len) is returned.
Warning
-
Nonedoes not mean thatget()does not return a value. The default value is completely ignored by this function. -
The value returned is the non-deduplicated length of the underlying Vector in storage.This means that any duplicate items are included.
Sourcepub fn migrate_keys<OldHasher1: StorageHasher, OldHasher2: StorageHasher, KeyArg1: EncodeLike<Key1>, KeyArg2: EncodeLike<Key2>>(
key1: KeyArg1,
key2: KeyArg2,
) -> Option<Value>
pub fn migrate_keys<OldHasher1: StorageHasher, OldHasher2: StorageHasher, KeyArg1: EncodeLike<Key1>, KeyArg2: EncodeLike<Key2>>( key1: KeyArg1, key2: KeyArg2, ) -> Option<Value>
Migrate an item with the given key1 and key2 from defunct OldHasher1 and
OldHasher2 to the current hashers.
If the key doesn’t exist, then it’s a no-op. If it does, then it returns its value.
Sourcepub fn remove_all(limit: Option<u32>) -> KillStorageResult
👎Deprecated: Use clear instead
pub fn remove_all(limit: Option<u32>) -> KillStorageResult
clear insteadRemove all values in the overlay and up to limit in the backend.
All values in the client overlay will be deleted, if there is some limit then up to
limit values are deleted from the client backend, if limit is none then all values in
the client backend are deleted.
Note
Calling this multiple times per block with a limit set leads always to the same keys being
removed and the same result being returned. This happens because the keys to delete in the
overlay are not taken into account when deleting keys in the backend.
Sourcepub fn clear(limit: u32, maybe_cursor: Option<&[u8]>) -> MultiRemovalResults
pub fn clear(limit: u32, maybe_cursor: Option<&[u8]>) -> MultiRemovalResults
Attempt to remove all items from the map.
Returns MultiRemovalResults to inform about the result. Once
the resultant maybe_cursor field is None, then no further items remain to be deleted.
NOTE: After the initial call for any given map, it is important that no further items
are inserted into the map. If so, then the map may not be empty when the resultant
maybe_cursor is None.
Limit
A limit must always be provided through in order to cap the maximum
amount of deletions done in a single call. This is one fewer than the
maximum number of backend iterations which may be done by this operation and as such
represents the maximum number of backend deletions which may happen.A limit of zero
implies that no keys will be deleted, though there may be a single iteration done.
Cursor
A cursor may be passed in to this operation with maybe_cursor. None should only be
passed once (in the initial call) for any given storage map. Subsequent calls
operating on the same map should always pass Some, and this should be equal to the
previous call result’s maybe_cursor field.
Sourcepub fn iter_values() -> PrefixIterator<Value> ⓘ
pub fn iter_values() -> PrefixIterator<Value> ⓘ
Iter over all value of the storage.
NOTE: If a value failed to decode because storage is corrupted then it is skipped.
Sourcepub fn translate_values<OldValue: Decode, F: FnMut(OldValue) -> Option<Value>>(
f: F,
)
pub fn translate_values<OldValue: Decode, F: FnMut(OldValue) -> Option<Value>>( f: F, )
Translate the values of all elements by a function f, in the map in no particular order.
By returning None from f for an element, you’ll remove it from the map.
NOTE: If a value fail to decode because storage is corrupted then it is skipped.
Warning
This function must be used with care, before being updated the storage still contains the
old type, thus other calls (such as get) will fail at decoding it.
Usage
This would typically be called inside the module implementation of on_runtime_upgrade.
Sourcepub fn try_append<KArg1, KArg2, Item, EncodeLikeItem>(
key1: KArg1,
key2: KArg2,
item: EncodeLikeItem,
) -> Result<(), ()>where
KArg1: EncodeLike<Key1> + Clone,
KArg2: EncodeLike<Key2> + Clone,
Item: Encode,
EncodeLikeItem: EncodeLike<Item>,
Value: StorageTryAppend<Item>,
pub fn try_append<KArg1, KArg2, Item, EncodeLikeItem>(
key1: KArg1,
key2: KArg2,
item: EncodeLikeItem,
) -> Result<(), ()>where
KArg1: EncodeLike<Key1> + Clone,
KArg2: EncodeLike<Key2> + Clone,
Item: Encode,
EncodeLikeItem: EncodeLike<Item>,
Value: StorageTryAppend<Item>,
Try and append the given item to the value in the storage.
Is only available if Value of the storage implements StorageTryAppend.
Sourceimpl<Prefix, Hasher1, Key1, Hasher2, Key2, Value, QueryKind, OnEmpty, MaxValues> StorageDoubleMap<Prefix, Hasher1, Key1, Hasher2, Key2, Value, QueryKind, OnEmpty, MaxValues>where
Prefix: StorageInstance,
Hasher1: StorageHasher + ReversibleStorageHasher,
Hasher2: StorageHasher + ReversibleStorageHasher,
Key1: FullCodec,
Key2: FullCodec,
Value: FullCodec,
QueryKind: QueryKindTrait<Value, OnEmpty>,
OnEmpty: Get<QueryKind::Query> + 'static,
MaxValues: Get<Option<u32>>,
impl<Prefix, Hasher1, Key1, Hasher2, Key2, Value, QueryKind, OnEmpty, MaxValues> StorageDoubleMap<Prefix, Hasher1, Key1, Hasher2, Key2, Value, QueryKind, OnEmpty, MaxValues>where
Prefix: StorageInstance,
Hasher1: StorageHasher + ReversibleStorageHasher,
Hasher2: StorageHasher + ReversibleStorageHasher,
Key1: FullCodec,
Key2: FullCodec,
Value: FullCodec,
QueryKind: QueryKindTrait<Value, OnEmpty>,
OnEmpty: Get<QueryKind::Query> + 'static,
MaxValues: Get<Option<u32>>,
Sourcepub fn iter_prefix(k1: impl EncodeLike<Key1>) -> PrefixIterator<(Key2, Value)> ⓘ
pub fn iter_prefix(k1: impl EncodeLike<Key1>) -> PrefixIterator<(Key2, Value)> ⓘ
Enumerate all elements in the map with first key k1 in no particular order.
If you add or remove values whose first key is k1 to the map while doing this, you’ll get
undefined results.
Sourcepub fn iter_prefix_from(
k1: impl EncodeLike<Key1>,
starting_raw_key: Vec<u8>,
) -> PrefixIterator<(Key2, Value)> ⓘ
pub fn iter_prefix_from( k1: impl EncodeLike<Key1>, starting_raw_key: Vec<u8>, ) -> PrefixIterator<(Key2, Value)> ⓘ
Enumerate all elements in the map with first key k1 after a specified starting_raw_key
in no particular order.
If you add or remove values whose first key is k1 to the map while doing this, you’ll get
undefined results.
Sourcepub fn iter_key_prefix(k1: impl EncodeLike<Key1>) -> KeyPrefixIterator<Key2> ⓘ
pub fn iter_key_prefix(k1: impl EncodeLike<Key1>) -> KeyPrefixIterator<Key2> ⓘ
Enumerate all second keys k2 in the map with the same first key k1 in no particular
order.
If you add or remove values whose first key is k1 to the map while doing this, you’ll get
undefined results.
Sourcepub fn iter_key_prefix_from(
k1: impl EncodeLike<Key1>,
starting_raw_key: Vec<u8>,
) -> KeyPrefixIterator<Key2> ⓘ
pub fn iter_key_prefix_from( k1: impl EncodeLike<Key1>, starting_raw_key: Vec<u8>, ) -> KeyPrefixIterator<Key2> ⓘ
Enumerate all second keys k2 in the map with the same first key k1 after a specified
starting_raw_key in no particular order.
If you add or remove values whose first key is k1 to the map while doing this, you’ll get
undefined results.
Sourcepub fn drain_prefix(k1: impl EncodeLike<Key1>) -> PrefixIterator<(Key2, Value)> ⓘ
pub fn drain_prefix(k1: impl EncodeLike<Key1>) -> PrefixIterator<(Key2, Value)> ⓘ
Remove all elements from the map with first key k1 and iterate through them in no
particular order.
If you add elements with first key k1 to the map while doing this, you’ll get undefined
results.
Sourcepub fn iter() -> PrefixIterator<(Key1, Key2, Value)> ⓘ
pub fn iter() -> PrefixIterator<(Key1, Key2, Value)> ⓘ
Enumerate all elements in the map in no particular order.
If you add or remove values to the map while doing this, you’ll get undefined results.
Sourcepub fn iter_from(
starting_raw_key: Vec<u8>,
) -> PrefixIterator<(Key1, Key2, Value)> ⓘ
pub fn iter_from( starting_raw_key: Vec<u8>, ) -> PrefixIterator<(Key1, Key2, Value)> ⓘ
Enumerate all elements in the map after a specified starting_raw_key in no particular
order.
If you add or remove values to the map while doing this, you’ll get undefined results.
Sourcepub fn iter_keys() -> KeyPrefixIterator<(Key1, Key2)> ⓘ
pub fn iter_keys() -> KeyPrefixIterator<(Key1, Key2)> ⓘ
Enumerate all keys k1 and k2 in the map in no particular order.
If you add or remove values to the map while doing this, you’ll get undefined results.
Sourcepub fn iter_keys_from(
starting_raw_key: Vec<u8>,
) -> KeyPrefixIterator<(Key1, Key2)> ⓘ
pub fn iter_keys_from( starting_raw_key: Vec<u8>, ) -> KeyPrefixIterator<(Key1, Key2)> ⓘ
Enumerate all keys k1 and k2 in the map after a specified starting_raw_key in no
particular order.
If you add or remove values to the map while doing this, you’ll get undefined results.
Sourcepub fn drain() -> PrefixIterator<(Key1, Key2, Value)> ⓘ
pub fn drain() -> PrefixIterator<(Key1, Key2, Value)> ⓘ
Remove all elements from the map and iterate through them in no particular order.
If you add elements to the map while doing this, you’ll get undefined results.
Sourcepub fn translate<O: Decode, F: FnMut(Key1, Key2, O) -> Option<Value>>(f: F)
pub fn translate<O: Decode, F: FnMut(Key1, Key2, O) -> Option<Value>>(f: F)
Translate the values of all elements by a function f, in the map in no particular order.
By returning None from f for an element, you’ll remove it from the map.
NOTE: If a value fail to decode because storage is corrupted then it is skipped.